"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.decrypt = exports.encrypt = exports.keyGeneration = exports.randomPlaintext = exports.scalarMultiplyVector = void 0;
const random_1 = require("./constants/random");
const const_1 = require("./constants/const");
function initMatrixDefault(x, y) {
    const matrix = new Array(x);
    for (let i = 0; i < x; i++) {
        matrix[i] = new Array(y);
    }
    return matrix;
}
function initMatrixRandom(x, y, q) {
    const matrix = new Array(x);
    for (let i = 0; i < x; i++) {
        matrix[i] = new Array(y);
        for (let j = 0; j < y; j++) {
            matrix[i][j] = nextInt(q);
        }
    }
    return matrix;
}
function transpose(A) {
    const A_x = A.length;
    const A_y = A[0].length;
    const C = initMatrixDefault(A_y, A_x);
    for (let i = 0; i < A_x; i++) {
        for (let j = 0; j < A_y; j++) {
            C[j][i] = A[i][j];
        }
    }
    return C;
}
function addMod(A, B, q) {
    checkDimensions(A, B);
    const A_x = A.length;
    const A_y = A[0].length;
    const C = initMatrixDefault(A_x, A_y);
    for (let i = 0; i < A_x; i++) {
        for (let j = 0; j < A_y; j++) {
            C[i][j] = (A[i][j] + B[i][j]) % q;
            if (C[i][j] < 0) {
                C[i][j] += q;
            }
        }
    }
    return C;
}
function vectorSubstract(a, b) {
    if (b.length != a.length) {
        alert("Vector length must agree");
        return;
    }
    const c = new Array(a.length);
    for (let i = 0; i < a.length; i++) {
        c[i] = a[i] - b[i];
    }
    return c;
}
function scalarMultiplyVector(s, v) {
    const c = new Array(v.length);
    for (let i = 0; i < v.length; i++) {
        c[i] = s * v[i];
    }
    return c;
}
exports.scalarMultiplyVector = scalarMultiplyVector;
function scalarMultiplyMod(s, A, q) {
    const A_x = A.length;
    const A_y = A[0].length;
    const C = initMatrixDefault(A_x, A_y);
    for (let i = 0; i < A_x; i++) {
        for (let j = 0; j < A_y; j++) {
            C[i][j] = (s * A[i][j]) % q;
            if (C[i][j] < 0) {
                C[i][j] += q;
            }
        }
    }
    return C;
}
function multiply(A, B) {
    const A_x = A.length;
    const A_y = A[0].length;
    const B_x = B.length;
    const B_y = B[0].length;
    if (B_x != A_y) {
        alert("Matrix inner dimensions must agree");
        return;
    }
    const C = initMatrixDefault(A_x, B_y);
    const Bcolj = new Array(A_y);
    for (let j = 0; j < B_y; j++) {
        for (let k = 0; k < A_y; k++) {
            Bcolj[k] = B[k][j];
        }
        for (let i = 0; i < A_x; i++) {
            const Arowi = A[i];
            let s = 0;
            for (let k = 0; k < A_y; k++) {
                if (Bcolj[k] == 0) {
                }
                else if (Bcolj[k] == 1) {
                    s += Arowi[k];
                }
                else if (Bcolj[k] == -1) {
                    s -= Arowi[k];
                }
                else {
                    s += Arowi[k] * Bcolj[k];
                }
            }
            C[i][j] = s;
        }
    }
    return C;
}
function multiplyMod(A, B, q) {
    const A_x = A.length;
    const A_y = A[0].length;
    const B_x = B.length;
    const B_y = B[0].length;
    if (B_x != A_y) {
        alert("Matrix inner dimensions must agree");
        return;
    }
    const C = initMatrixDefault(A_x, B_y);
    const Bcolj = new Array(A_y);
    for (let j = 0; j < B_y; j++) {
        for (let k = 0; k < A_y; k++) {
            Bcolj[k] = B[k][j];
        }
        for (let i = 0; i < A_x; i++) {
            const Arowi = A[i];
            let s = 0;
            for (let k = 0; k < A_y; k++) {
                if (Bcolj[k] == 0) {
                }
                else if (Bcolj[k] == 1) {
                    s += Arowi[k];
                }
                else if (Bcolj[k] == -1) {
                    s -= Arowi[k];
                }
                else {
                    s += Arowi[k] * Bcolj[k];
                }
            }
            C[i][j] = s % q;
            if (C[i][j] < 0) {
                C[i][j] += q;
            }
        }
    }
    return C;
}
function encVectorMultiplyMatrix(a, B) {
    const A_y = a.length;
    const B_x = B.length;
    const B_y = B[0].length;
    if (B_x != A_y) {
        alert("Matrix inner dimensions must agree");
        return;
    }
    const v = new Array(B_y);
    for (let j = 0; j < B_y; j++) {
        v[j] = 0;
    }
    for (let i = 0; i < A_y; i++) {
        const Browi = B[i];
        for (let j = 0; j < B_y; j++) {
            if (a[i] == 0) {
            }
            else if (a[i] == 1) {
                v[j] += Browi[j];
            }
            else if (a[i] == -1) {
                v[j] -= Browi[j];
            }
            else {
                v[j] += a[i] * Browi[j];
            }
        }
    }
    return v;
}
function decVectorMultiplyMatrix(a, B) {
    const A_y = a.length;
    const B_x = B.length;
    const B_y = B[0].length;
    if (B_x != A_y) {
        alert("Matrix inner dimensions must agree");
        return;
    }
    const v = new Array(B_y);
    for (let j = 0; j < B_y; j++) {
        v[j] = 0;
    }
    for (let i = 0; i < A_y; i++) {
        const Browi = B[i];
        for (let j = 0; j < B_y; j++) {
            v[j] += a[i] * Browi[j];
        }
    }
    return v;
}
function mod(A, q) {
    const A_x = A.length;
    const A_y = A[0].length;
    if (q <= 0) {
        alert("modulus not positive");
        return;
    }
    for (let i = 0; i < A_x; i++) {
        for (let j = 0; j < A_y; j++) {
            A[i][j] %= q;
            if (A[i][j] < 0) {
                A[i][j] += q;
            }
        }
    }
}
function checkDimensions(A, B) {
    const A_x = A.length;
    const A_y = A[0].length;
    const B_x = B.length;
    const B_y = B[0].length;
    if (B_x != A_x || B_y != A_y) {
        alert("Matrix dimensions must agree");
        return;
    }
}
function nextInt(q) {
    return Math.floor(Math.random() * q);
}
function rangeValue(low, high) {
    return Math.floor(Math.random() * (high - low + 1) + low);
}
function shuffle(arr) {
    const arr2 = arr.slice();
    for (let j, x, i = arr2.length; i; j = Math.random() * i, x = arr2[--i], arr2[i] = arr2[j], arr2[j] = x)
        ;
    return arr2;
}
function randomPlaintext() {
    const plaintext = new Array(const_1.l);
    for (let i = 0; i < const_1.l; i++) {
        plaintext[i] = nextInt(2);
    }
    return plaintext;
}
exports.randomPlaintext = randomPlaintext;
function keyGeneration() {
    const amatrix = initMatrixRandom(const_1.m, const_1.n, const_1.q);
    const smatrix = random_1.ss;
    const ematrix = random_1.ee;
    let bmatrix = multiply(amatrix, smatrix);
    bmatrix = addMod(bmatrix, ematrix, const_1.q);
    return { pk: { a: amatrix, b: bmatrix }, sk: { s: smatrix } };
}
exports.keyGeneration = keyGeneration;
function encrypt(msg, pk) {
    const amatrix = pk.a;
    const bmatrix = pk.b;
    const rvector_transpose = random_1.rr;
    const c1_prime_transpose = encVectorMultiplyMatrix(rvector_transpose, amatrix);
    const c2_prime_transpose = encVectorMultiplyMatrix(rvector_transpose, bmatrix);
    const c1_transpose = new Array(const_1.n);
    for (let i = 0; i < const_1.n; i++) {
        c1_prime_transpose[i] = c1_prime_transpose[i] % const_1.q;
        c1_transpose[i] = Math.round(0.25 * c1_prime_transpose[i]) % const_1.p;
        if (c1_transpose[i] < 0) {
            c1_transpose[i] += const_1.p;
        }
    }
    const c2_transpose = new Array(const_1.l);
    for (let i = 0; i < const_1.l; i++) {
        c2_prime_transpose[i] = c2_prime_transpose[i] % const_1.q;
        c2_transpose[i] = Math.round(msg[i] + 0.25 * c2_prime_transpose[i]) % const_1.p;
        if (c2_transpose[i] < 0) {
            c2_transpose[i] += const_1.p;
        }
    }
    return { c1: c1_transpose, c2: c2_transpose };
}
exports.encrypt = encrypt;
function decrypt(encrypted, sk) {
    const c1vector_transpose = encrypted.c1;
    const c2vector_transpose = encrypted.c2;
    const smatrix = sk.s;
    const c1TS = decVectorMultiplyMatrix(c1vector_transpose, smatrix);
    const resultvector_transpose = vectorSubstract(c2vector_transpose, c1TS);
    const resultvector = new Array(const_1.l);
    for (let i = 0; i < const_1.l; i++) {
        resultvector[i] = Math.round(0.0078125 * resultvector_transpose[i]) % const_1.t;
        if (resultvector[i] < 0) {
            resultvector[i] += const_1.t;
        }
    }
    return resultvector;
}
exports.decrypt = decrypt;
//# sourceMappingURL=main.js.map